package cn.ninegame.genericframework.basic;

import android.text.TextUtils;

import java.math.BigInteger;
import java.util.HashMap;

import cn.ninegame.genericframework.module.IModuleEntry;
import cn.ninegame.genericframework.module.ModuleCenter;
import cn.ninegame.genericframework.tools.CRC16;

/**
 * Created by lingcong on 2015/3/18.
 * 1、添加，删除，替换组件
 * 2、组件的加载接口
 * 3、管理和保存组件
 * 4、管理IController的生命周期
 */
public class ControllerCenter  {

    private Environment mEnv = null;
    private  MsgBroker mMsgBroker = null;
    private ModuleCenter mModuleCenter = null;

    //建立ControllerID与ModuleID的映射关系。
    private HashMap<String,String> mControllerModuleIDMap = new HashMap<String, String>(2);

    //建立ControllerID与Controller的映射关系
    private HashMap<String,IController> mControllerIDMap = new HashMap<String, IController>(2);

    //建立ControllerID与Controller的映射关系
    private HashMap<String,Object> mControllerObjectMap = new HashMap<String, Object>(2);

    public  ControllerCenter() {

    }

    public  void setEnvironment(Environment env) {
        mEnv = env;
    }

    public void setModuleCenter(ModuleCenter moduleCenter) {
        mModuleCenter = moduleCenter;
    }

    public Environment getEnviroment(){
        return mEnv;
    }

    public void setMsgBroker(MsgBroker msgBroker) {
        mMsgBroker = msgBroker;
    }

    public void registerModuleControllers(String moduleID,ControllerData[] controllerDatas) {
        if (controllerDatas == null || controllerDatas.length == 0) {
            return;
        }

        for (int i = 0;i < controllerDatas.length ; i++) {
            if(TextUtils.isEmpty(controllerDatas[i].id)) {
                continue;
            }
            if(controllerDatas[i].messages == null || controllerDatas[i].messages.length == 0) {
                continue;
            }

            mControllerIDMap.put(controllerDatas[i].id, null);
            mMsgBroker.registerMessageHandler(controllerDatas[i].id,controllerDatas[i].messages);

            mControllerModuleIDMap.put(controllerDatas[i].id,moduleID);
        }

    }

    public IModuleEntry getModuleEntryBy(String messageHandlerID){
        return mModuleCenter.getModuleEntry(mControllerModuleIDMap.get(messageHandlerID));
    }

    public IMessageHandler getMessageHandler(String messageHandlerID) {
        IController controller = null;
        Object controllerObject  = null;
        if(TextUtils.isEmpty(messageHandlerID)) {
            return  null;
        }
        controller = mControllerIDMap.get(messageHandlerID);
        controllerObject = mControllerObjectMap.get(messageHandlerID);
        if(controller == null || controllerObject==null) {
            IModuleEntry moduleEntry = mModuleCenter.getModuleEntry(mControllerModuleIDMap.get(messageHandlerID));
            if(moduleEntry == null) {
                return  null;
            }
            controllerObject = moduleEntry.loadControllerObject(messageHandlerID);
            if(controllerObject instanceof IController) {
                controller = (IController) controllerObject;
                controller.setEnvironment(mEnv);
                controller.onInit();
                mControllerIDMap.put(messageHandlerID, controller);
            }
            mControllerObjectMap.put(messageHandlerID,controllerObject);
        }
        return controller;
    }

    public Object getMessageHandlerObject(String messageHandlerID) {
        IController controller = null;
        Object controllerObject = null;
        if(TextUtils.isEmpty(messageHandlerID)) {
            return  null;
        }
        controller = mControllerIDMap.get(messageHandlerID);
        controllerObject = mControllerObjectMap.get(messageHandlerID);
        if(controller == null || controllerObject==null) {
            IModuleEntry moduleEntry = mModuleCenter.getModuleEntry(mControllerModuleIDMap.get(messageHandlerID));
            if(moduleEntry == null) {
                return  null;
            }
            controllerObject = moduleEntry.loadControllerObject(messageHandlerID);
            if(controllerObject instanceof IController) {
                controller = (IController) controllerObject;
                controller.setEnvironment(mEnv);
                controller.onInit();
                mControllerIDMap.put(messageHandlerID, controller);
            }
            //controller = moduleEntry.loadController(messageHandlerID);
            mControllerObjectMap.put(messageHandlerID,controllerObject);
        }
        return controllerObject;
    }

    public int crcGumdamMessageIdCode(String msg) {
        StringBuffer stringBuffer = new StringBuffer();
        stringBuffer.append("_");
        stringBuffer.append(msg);
        stringBuffer.append("_");
        int crc16 = CRC16.calcCrc16(stringBuffer.toString().getBytes());
        //00 default,01 notification type,
        return new BigInteger(String.format("0002%04x",crc16), 16).intValue();
        //return stringBuffer.toString().hashCode();
    }

}
