package cn.ninegame.genericframework.basic;

import android.os.Bundle;
import android.os.Handler;
import android.os.Looper;
import android.os.Message;


import cn.ninegame.genericframework.module.CacheOperation.DialogOperation;
import cn.ninegame.genericframework.module.CacheOperation.FragmentOperation;
import cn.ninegame.genericframework.module.CacheOperation.MultiFragmentOperation;
import cn.ninegame.genericframework.module.CacheOperation.Operation;

/**
 * Created by lingcong on 2016/11/18.
 */
public abstract class EnvironmentInterceptor implements Handler.Callback {

    private Operation mCacheOperation = null;
    private Environment mEnv = null;
    private Handler mMainHandler  = null;

    private static final int MSG_INTERCEPT_FRGMENT = 1;

    public EnvironmentInterceptor() {
        if (mMainHandler == null) {
            mMainHandler = new Handler(Looper.getMainLooper(),this);
        }
    }

    void setEnvironment(Environment env) {
        mEnv = env;
    }

    public abstract boolean needInterceptFragment(String fragment, Bundle param);

    public abstract void onInterceptFragments(String[] fragments, Bundle[] params);

    void interceptMutilFragment(String[] fragments,Bundle[] params,int[] modes) {
        MultiFragmentOperation mfo = new MultiFragmentOperation(fragments,params,modes);
        interceptFragmentOperation(mfo);
    }

    void interceptFragment( String fragment, Bundle param, IResultListener listener, boolean useAnim, int mode) {
        FragmentOperation fo = new FragmentOperation(fragment,param,listener,useAnim,mode);
        interceptFragmentOperation(fo);
    }

    void interceptFragmentWithFlags(String fragmentID, Bundle param, boolean useAnim, int intentFlag) {
        FragmentOperation fo = new FragmentOperation(fragmentID,param,useAnim,intentFlag);
        interceptFragmentOperation(fo);
    }

    void interceptDialogFragment(String fragmentID, Bundle param, IResultListener listener ) {
        DialogOperation dlgop = new DialogOperation(fragmentID,param,listener);
        interceptFragmentOperation(dlgop);
    }

    private void interceptFragmentOperation(Operation op) {
        mCacheOperation = op;

        Message message = Message.obtain();
        message.what = MSG_INTERCEPT_FRGMENT;
        message.obj = op;

        mMainHandler.sendMessage(message);
    }

    public void continueOperation() {
        if (mCacheOperation == null) {
            return;
        }

        if(mCacheOperation instanceof FragmentOperation){
            FragmentOperation fo = (FragmentOperation)mCacheOperation;
            switch (fo.type){
                case FragmentOperation.OPERATION_TYPE_FRAGMENT_WITH_FLAG:
                    mEnv.startFragmentWithFlagNoIntercept(fo.fragmentID,fo.param,fo.useAnim,fo.flag);
                    break;
                case FragmentOperation.OPERATION_TYPE_FRAGMENT_WITH_MODE:
                    mEnv.startFragmentForResultNoIntercept(fo.fragmentID, fo.param, fo.listener,fo.useAnim,fo.mode);
                    break;
            }
        }else if(mCacheOperation instanceof MultiFragmentOperation){
            MultiFragmentOperation mfo = (MultiFragmentOperation)mCacheOperation;
            mEnv.startFragmentsNoIntercept(mfo.fragmentIDs, mfo.params, mfo.modes);
        }else if (mCacheOperation instanceof DialogOperation) {
            DialogOperation dlgOp = (DialogOperation)mCacheOperation;
            mEnv.startDialogFragmentNoIntercept(dlgOp.fragmentID, dlgOp.param, dlgOp.listener);
        }

    }

    @Override
    public boolean handleMessage(Message msg) {
        if (msg.what != MSG_INTERCEPT_FRGMENT) {
            return  false;
        }

        String[] fragments = null;
        Bundle[] params = null;

        Operation operation = (Operation) msg.obj;

        if (operation instanceof FragmentOperation) {
            fragments = new String[]{((FragmentOperation) operation).fragmentID};
            params = new Bundle[]{((FragmentOperation) operation).param};
        }else if (operation instanceof MultiFragmentOperation) {
            fragments = ((MultiFragmentOperation) operation).fragmentIDs;
            params = ((MultiFragmentOperation) operation).params;
        }else if (operation instanceof DialogOperation) {
            fragments = new String[]{((DialogOperation) operation).fragmentID};
            params = new Bundle[]{((DialogOperation) operation).param};
        }

        if (fragments != null) {
            onInterceptFragments(fragments,params);
        }

        return true;
    }
}
