package cn.ninegame.genericframework.module;

import org.json.JSONArray;
import org.json.JSONObject;
import cn.ninegame.genericframework.basic.ControllerData;
import cn.ninegame.genericframework.basic.FrameworkService;
import cn.ninegame.genericframework.basic.GundamFragment;
import cn.ninegame.genericframework.basic.GundamMessage;
import cn.ninegame.genericframework.basic.GundamNotification;

import java.io.IOException;
import java.io.InputStream;
import java.util.ArrayList;


/**
 * ****************************************************************************
 * Copyright @ 2009 - 2015 www.9game.cn All Rights Reserved
 * <p/>
 * Creation    : 2016-01-26
 * Author      : lihq@ucweb.com
 * Description : Tell me what does this class do
 * ****************************************************************************
 */
public class JsonModuleManifestParser implements IManifestParser {

    @Override
    public IModuleManifest[] parser(InputStream inputStream) {
        int lenght = 0;
        try {
            lenght = inputStream.available();
            byte[] data = new byte[lenght];
            inputStream.read(data);

            return parser(data);
        } catch (IOException e) {
            FrameworkService.addDebugLog("ModuleLoader","parserModuleManifest",e);
            e.printStackTrace();
        }

        return null;
    }

    public IModuleManifest[] parser(byte[] data) {
        try{
            //ModuleConfig moduleConfig = JSON.parseObject(data, ModuleConfig.class);
            ModuleConfig moduleConfig = loadModuleConfig(data);
            if(moduleConfig==null) return null;
            ArrayList<Module> modules = moduleConfig.getModules();
            ModuleManifestDynamic[] manifestArray = new ModuleManifestDynamic[modules.size()];
            for (int i=0 ; i < modules.size(); i++){
                ModuleManifestDynamic manifest = new ModuleManifestDynamic();
                manifest.setControllerDatas(modules.get(i).getControllers());
                manifest.setFragmentIDs(modules.get(i).getFragments());
                manifest.setModuleApplication(modules.get(i).getApplication());
                manifest.setNotifications(modules.get(i).getNotifications());

                ModuleInfo moduleInfo = new ModuleInfo();
                moduleInfo.setID(modules.get(i).getId());
                moduleInfo.setVersion(modules.get(i).getVersion());
                moduleInfo.setSignature(modules.get(i).getSignature());
                moduleInfo.setAndroidLibrary(modules.get(i).isAndroidLibrary());
                manifest.setModuleInfo(moduleInfo);
                manifestArray[i] = manifest;
            }
            return manifestArray;

        }catch (Exception e){
            FrameworkService.addDebugLog("ModuleLoader","parserModuleManifest",e);
            e.printStackTrace();
        }

        return null;
    }


    public ArrayList<Module> parserForModule(InputStream inputStream) {
        ArrayList<Module> modules = null;
        try {
            int lenght = inputStream.available();
            byte[] data = new byte[lenght];
            inputStream.read(data);
            ModuleConfig moduleConfig = loadModuleConfig(data);
            //ModuleConfig moduleConfig = JSON.parseObject(data, ModuleConfig.class);
            if(moduleConfig!=null){
                modules = moduleConfig.getModules();
            }
        }catch (Exception e){
            FrameworkService.addDebugLog("ModuleLoader","parserForModule",e);
            e.printStackTrace();
        }

        return modules;
    }

    public ModuleConfig loadModuleConfig(byte[] data){
        String strVal;
        char[] chars = allocateChars(data.length);
        int chars_len = decodeUTF8(data, 0, data.length, chars);
        strVal = new String(chars, 0, chars_len);
        ModuleConfig moduleConfig = null;
        try{
            JSONObject jsonObject  = new JSONObject(strVal);
            moduleConfig = new ModuleConfig();
            ArrayList<Module> moduleArray= new ArrayList<>();
            JSONArray modules = jsonObject.getJSONArray("modules");
            for(int i=0;i<modules.length();i++){
                JSONObject moduleJsonObject = modules.getJSONObject(i);
                Module moduleObject = new Module();
                moduleObject.setApplication(moduleJsonObject.getString("application"));
                moduleObject.setId(moduleJsonObject.getString("id"));
                moduleObject.setSignature(moduleJsonObject.optString("signature"));
                moduleObject.setVersion(moduleJsonObject.getString("version"));
                if(moduleJsonObject.has("androidLibrary")){
                    moduleObject.setAndroidLibrary(moduleJsonObject.getBoolean("androidLibrary"));
                }
                if(moduleJsonObject.has("controllers")){
                    JSONArray controller = moduleJsonObject.getJSONArray("controllers");
                    ControllerData[] controllerDataArray = new ControllerData[controller.length()];
                    for(int j=0;j<controller.length();j++){
                        ControllerData controllerData = new ControllerData();
                        controllerData.setId(controller.getJSONObject(j).getString("id"));
                        JSONArray message = controller.getJSONObject(j).getJSONArray("messages");
                        GundamMessage[] messageArray = new GundamMessage[message.length()];
                        for (int k = 0; k < message.length(); k++) {
                            JSONObject messageJsonObject = message.getJSONObject(k);
                            GundamMessage gundamMessage = new GundamMessage();
                            gundamMessage.id = messageJsonObject.getInt("id");
                            gundamMessage.name = messageJsonObject.getString("name");
                            gundamMessage.threadMode = messageJsonObject.getString("threadMode");
                            if(messageJsonObject.has("bindMethod")){
                                gundamMessage.method = messageJsonObject.getString("bindMethod");
                            }
                            messageArray[k] = gundamMessage;
                        }
                        controllerData.setMessages(messageArray);
                        controllerDataArray[j] = controllerData;
                    }
                    moduleObject.setControllers(controllerDataArray);
                }
                if(moduleJsonObject.has("fragments")){
                    JSONArray fragments = moduleJsonObject.getJSONArray("fragments");
                    GundamFragment[] fragmentsArray = new GundamFragment[fragments.length()];
                    for (int k = 0; k < fragments.length(); k++) {
                        GundamFragment gundamFragment= new GundamFragment();
                        gundamFragment.name = fragments.getJSONObject(k).getString("name");
                        gundamFragment.className = fragments.getJSONObject(k).getString("className");
                        fragmentsArray[k] = gundamFragment;
                    }
                    moduleObject.setFragments(fragmentsArray);
                }
                if(moduleJsonObject.has("notifications")){
                    JSONArray notifications = moduleJsonObject.getJSONArray("notifications");
                    GundamNotification[] notificationsArray = new GundamNotification[notifications.length()];
                    for (int k = 0; k < notifications.length(); k++) {
                        JSONObject messageJsonObject = notifications.getJSONObject(k);
                        GundamNotification gundamNotification = new GundamNotification();
                        gundamNotification.id = messageJsonObject.getInt("id");
                        gundamNotification.name = messageJsonObject.getString("name");
                        gundamNotification.threadMode = messageJsonObject.getString("threadMode");
                        if(messageJsonObject.has("bindMethod")){
                            gundamNotification.method = messageJsonObject.getString("bindMethod");
                        }
                        notificationsArray[k] = gundamNotification;
                    }
                    moduleObject.setNotifications(notificationsArray);
                }
                moduleArray.add(moduleObject);
            }
            moduleConfig.setModules(moduleArray);
        }catch (Exception ex){
            FrameworkService.addDebugLog("ModuleLoader","loadModuleConfig",ex);
            ex.printStackTrace();
        }
        return moduleConfig;
    }

    private final static ThreadLocal<char[]> charsLocal = new ThreadLocal<char[]>();
    private static char[] allocateChars(int length) {
        char[] chars = charsLocal.get();

        if (chars == null) {
            if (length <= 1024 * 64) {
                chars = new char[1024 * 64];
                charsLocal.set(chars);
            } else {
                chars = new char[length];
            }
        } else if (chars.length < length) {
            chars = new char[length];
        }

        return chars;
    }

    public static int decodeUTF8(byte[] sa, int sp, int len, char[] da) {
        final int sl = sp + len;
        int dp = 0;
        int dlASCII = Math.min(len, da.length);

        // ASCII only optimized loop
        while (dp < dlASCII && sa[sp] >= 0)
            da[dp++] = (char) sa[sp++];

        while (sp < sl) {
            int b1 = sa[sp++];
            if (b1 >= 0) {
                // 1 byte, 7 bits: 0xxxxxxx
                da[dp++] = (char) b1;
            } else if ((b1 >> 5) == -2 && (b1 & 0x1e) != 0) {
                // 2 bytes, 11 bits: 110xxxxx 10xxxxxx
                if (sp < sl) {
                    int b2 = sa[sp++];
                    if ((b2 & 0xc0) != 0x80) { // isNotContinuation(b2)
                        return -1;
                    } else {
                        da[dp++] = (char) (((b1 << 6) ^ b2)^
                                (((byte) 0xC0 << 6) ^
                                        ((byte) 0x80 << 0)));
                    }
                    continue;
                }
                return -1;
            } else if ((b1 >> 4) == -2) {
                // 3 bytes, 16 bits: 1110xxxx 10xxxxxx 10xxxxxx
                if (sp + 1 < sl) {
                    int b2 = sa[sp++];
                    int b3 = sa[sp++];
                    if ((b1 == (byte) 0xe0 && (b2 & 0xe0) == 0x80) //
                            || (b2 & 0xc0) != 0x80 //
                            || (b3 & 0xc0) != 0x80) { // isMalformed3(b1, b2, b3)
                        return -1;
                    } else {
                        char c = (char)((b1 << 12) ^
                                (b2 <<  6) ^
                                (b3 ^
                                        (((byte) 0xE0 << 12) ^
                                                ((byte) 0x80 <<  6) ^
                                                ((byte) 0x80 <<  0))));
                        if (Character.isSurrogate(c)) {
                            return -1;
                        } else {
                            da[dp++] = c;
                        }
                    }
                    continue;
                }
                return -1;
            } else if ((b1 >> 3) == -2) {
                // 4 bytes, 21 bits: 11110xxx 10xxxxxx 10xxxxxx 10xxxxxx
                if (sp + 2 < sl) {
                    int b2 = sa[sp++];
                    int b3 = sa[sp++];
                    int b4 = sa[sp++];
                    int uc = ((b1 << 18) ^
                            (b2 << 12) ^
                            (b3 <<  6) ^
                            (b4 ^
                                    (((byte) 0xF0 << 18) ^
                                            ((byte) 0x80 << 12) ^
                                            ((byte) 0x80 <<  6) ^
                                            ((byte) 0x80 <<  0))));
                    if (((b2 & 0xc0) != 0x80 || (b3 & 0xc0) != 0x80 || (b4 & 0xc0) != 0x80) // isMalformed4
                            ||
                            // shortest form check
                            !Character.isSupplementaryCodePoint(uc)) {
                        return -1;
                    } else {
                        da[dp++] = Character.highSurrogate(uc);
                        da[dp++] = Character.lowSurrogate(uc);
                    }
                    continue;
                }
                return -1;
            } else {
                return -1;
            }
        }
        return dp;
    }

}
