package cn.ninegame.genericframework.module;

import android.app.Application;
import android.content.Context;
import android.content.ContextWrapper;
import android.content.res.AssetManager;
import android.content.res.Resources;
import android.os.Build;
import android.view.LayoutInflater;

import cn.ninegame.genericframework.exception.LoadContextException;
import cn.ninegame.genericframework.tools.ReflectHelper;

import java.lang.reflect.Method;

/**
 * Context for module, using module own Resources/Assets/LayoutInflater.
 *
 * Created by chenkai on 2016/1/19.
 */
public class ModuleContext extends ContextWrapper {

    private final Resources mResources;
    private final LayoutInflater mInflater;
    private final int mThemeResource;
    private final Resources.Theme mTheme;

    ModuleContext(Context base, Resources newRes) throws LoadContextException {
        super(base);
        mResources = newRes;

        try {
            mTheme = mResources.newTheme();
            Resources.Theme theme = getBaseContext().getTheme();
            if (theme != null) {
                mTheme.setTo(theme);
            }

            if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.ICE_CREAM_SANDWICH) {
                Method method = ReflectHelper.findMethod(Resources.class, "selectDefaultTheme", new Class[]{int.class, int.class});
                mThemeResource = (Integer) method.invoke(Resources.class, 0, getApplicationInfo().targetSdkVersion);
                mTheme.applyStyle(mThemeResource, true);
            } else {
                mThemeResource = 0;
            }

            mInflater = LayoutInflater.from(getBaseContext()).cloneInContext(this);
        } catch (Exception e) {
            throw new LoadContextException(e);
        }
    }

    @Override
    public AssetManager getAssets() {
        return mResources.getAssets();
    }

    @Override
    public Resources getResources() {
        return mResources;
    }

    public int getThemeResId() {
        return mThemeResource;
    }

    @Override
    public Resources.Theme getTheme() {
        return mTheme;
    }

    @Override
    public Object getSystemService(String name) {
        if (LAYOUT_INFLATER_SERVICE.equals(name)) {
            return mInflater;
        }
        return getBaseContext().getSystemService(name);
    }
}
