package cn.ninegame.genericframework.module;

import java.util.LinkedList;
import java.util.List;

import cn.ninegame.genericframework.basic.Environment;
import cn.ninegame.genericframework.tools.DeviceInfoUtil;
import cn.ninegame.genericframework.tools.LOG;

/**
 * <p>Title: ucweb</p>
 * <p/>
 * <p>Description: </p>
 * 模块预加载器
 * <p>Copyright: Copyright (c) 2016</p>
 * <p/>
 * <p>Company: ucweb.com</p>
 *
 * @author ZhuZhiFan (zhifan.zzf@alibaba-inc.com)
 * @date 2016/5/20
 */
public class ModulePreloader extends Thread{
    private static final String TAG = "ModulePreloader";
    // 模块加载的最低时间间隔
    private static final long DEFAULT_INTERVAL = 500;
    // 判断 CPU 是否处于忙时的最低阈值
    private static final float CPU_BUSY_THRESHOLD = 0.4f;
    // 触发模块加载的消息列表
    private LinkedList<Integer> mMsgs = new LinkedList<Integer>();
    private ModuleCenter mModuleCenter;
    private Environment mEnvironment;
    // 模块加载的时间间隔
    private long mInterval;
    // 判断 CPU 是否处于忙时的阈值
    private float mCpuBusyThreshold;

    public ModulePreloader(Environment environment, ModuleCenter moduleCenter, List<Integer> msgs, long interval, float cpuBusyThreshold) {
        LOG.i(TAG, String.format("ModulePreloader init, msg length=%d, interval=%d, cpuBusyThreshold=%f", msgs.size(), interval, cpuBusyThreshold));
        mEnvironment = environment;
        mModuleCenter = moduleCenter;
        mMsgs.addAll(msgs);
        mInterval = Math.max(interval, DEFAULT_INTERVAL);
        mCpuBusyThreshold = Math.max(cpuBusyThreshold, CPU_BUSY_THRESHOLD);
    }

    @Override
    public void run() {
        while (mMsgs.size() > 0) {
            if (!mModuleCenter.hasModuleLoading()) {
                if (DeviceInfoUtil.getProcessCpuUsage() <= mCpuBusyThreshold) {
                    Integer msg = mMsgs.getFirst();
                    mEnvironment.sendMessage(msg);
                    mMsgs.removeFirst();
                }
            }

            try {
                Thread.sleep(mInterval);
            } catch (InterruptedException e) {
                e.printStackTrace();
            }
        }

    }


}
