package cn.ninegame.genericframework.tools;


import java.io.BufferedInputStream;
import java.io.BufferedOutputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.util.Properties;

/**
 * ****************************************************************************
 * Copyright @ 2009 - 2015 www.9game.cn All Rights Reserved
 * <p/>
 * Creation    : 2016-03-10
 * Author      : lihq@ucweb.com
 * Description : Tell me what does this class do
 * ****************************************************************************
 */
public class FileUtil {

    public static boolean deleteDirectory(File file) {
        boolean success = false;
        try {
            if (file != null && file.exists()) {
                if (file.isDirectory()) {
                    File[] files = file.listFiles();
                    if (files != null) {
                        for (File subFile : files) {
                            if (subFile.isDirectory()) {
                                deleteDirectory(subFile);
                            } else {
                                subFile.delete();
                            }
                        }
                    }
                }
                success = file.delete();
            }
        } catch (Exception e) {
            e.printStackTrace();
        }
        return success;
    }


    /**
     * 递归删除文件和文件夹
     *
     * @param file
     *            要删除的根目录
     */
    public static void deleteFiles(File file) {
        if (file.exists()) {
            if (file.isFile()) {
                file.delete();
                return;
            }

            if (file.isDirectory()) {
                File[] childFiles = file.listFiles();

                if (childFiles == null || childFiles.length == 0) {
                    file.delete();
                    return;
                }

                for (File objFile : childFiles) {
                    deleteFiles(objFile);
                }

                file.delete();
            }
        }
    }

    private static void deleteFile(File file){
        if(file.exists()){
            if(file.isFile()){
                file.delete();
            }else if(file.isDirectory()){
                File files[] = file.listFiles();
                int len = files != null ? files.length : 0;
                for(int i=0;i<len;i++){
                    deleteFile(files[i]);
                }
            }
            file.delete();
        }
    }

    /**
     * 写入properties信息
     *
     * @param file       Property 文件
     * @param key  属性名称
     * @param value 属性值
     * @return
     */
    public static boolean writeProperty(File file, String key, String value) {
        Properties prop = new Properties();
        InputStream fis = null;
        OutputStream fos = null;
        try {
            fis = new FileInputStream(file);
            prop.load(fis);
            fos = new FileOutputStream(file);
            prop.setProperty(key, value);
            prop.store(fos, "Update '" + key + "' value");
            return true;
        } catch (IOException e) {
            e.printStackTrace();
            return false;
        }finally {
            IOUtils.safeClose(fis);
            IOUtils.safeClose(fos);
        }
    }

    /**
     * 根据key读取value
     *
     * @param file
     * @param key
     * @return
     */
    public static String readProperty(File file, String key) {
        Properties props = new Properties();
        FileInputStream fileInputStream = null;
        InputStream in = null;
        try {
            fileInputStream = new FileInputStream(file);
            in = new BufferedInputStream(fileInputStream);
            props.load(in);
            String value = props.getProperty(key);
            return value;
        } catch (Exception e) {
            e.printStackTrace();
            return "";
        }finally {
            IOUtils.safeClose(fileInputStream);
            IOUtils.safeClose(in);
        }
    }

    /**
     *
     * @param path
     *            ：文件路径
     * @param append
     *            ：若存在是否插入原文件
     * @return
     */
    public static File createNewFile(String path, boolean append) {
        File newFile = new File(path);
        if (!append) {
            if (newFile.exists()) {
                newFile.delete();
            }
        }
        if (!newFile.exists()) {
            try {
                File parent = newFile.getParentFile();
                if (parent != null && !parent.exists()) {
                    parent.mkdirs();
                }
                newFile.createNewFile();
            } catch (Exception e) {
                e.printStackTrace();
            }
        }
        return newFile;
    }

    public static File createNewFile(String path) {
        return createNewFile(path, false);
    }

    /**
     * Move文件到目录,重命名文件
     * @param resFile
     * @param objFolderFile
     * @param objName
     * @throws IOException
     */
    public static void moveAndRename(File resFile, File objFolderFile, String objName)
            throws IOException {
        copy(resFile, objFolderFile, objName);
    }

    /**
     * Copy文件到指定目录,重命名文件
     *
     * @param resFile
     * @param objFolderFile
     * @param targetName
     * @throws IOException
     */
    public static void copy(File resFile, File objFolderFile, String targetName)
            throws IOException {
        if (!resFile.exists())
            return;
        if (!objFolderFile.exists())
            objFolderFile.mkdirs();
        if (resFile.isFile()) {
            File objFile = new File(objFolderFile.getPath() + File.separator
                    + targetName);
            InputStream ins = new FileInputStream(resFile);
            FileOutputStream outs = new FileOutputStream(objFile);
            byte[] buffer = new byte[2* 1024 * 1024];
            int length;
            while ((length = ins.read(buffer)) != -1) {
                outs.write(buffer, 0, length);
            }
            ins.close();
            outs.flush();
            outs.close();
        } else {
            String objFolder = objFolderFile.getPath() + File.separator
                    + resFile.getName();
            File _objFolderFile = new File(objFolder);
            _objFolderFile.mkdirs();
            for (File sf : resFile.listFiles()) {
                copy(sf, new File(objFolder), targetName);
            }
        }
    }

    /**
     * 将文件转成byte数组
     * @param file
     * @return
     */
    public static byte[] fileToByte(File file){
        if (file != null && file.exists()){
            InputStream inputStream = null;
            try {
                inputStream = new FileInputStream(file);
                int length = inputStream.available();
                byte[] data = new byte[length];
                inputStream.read(data);
                return data;
            } catch (Exception e) {
                e.printStackTrace();
            }
        }
        return null;
    }

    /**
     * 将byte数组写到文件中
     * @param path
     * @param buff
     * @return
     */
    public static boolean writeFile(String path, byte[] buff) {
        OutputStream os = null;
        try {
            File file = new File(path);
            if (file.exists()) {
                file.delete();
            }
            file.createNewFile();

            os = new BufferedOutputStream(new FileOutputStream(file));
            os.write(buff);
            os.flush();
            return true;
        } catch (IOException ex) {
            ex.printStackTrace();
        } finally {
            try {
                os.close();
            } catch (IOException ex) {
                ex.printStackTrace();
            }
        }
        return false;
    }
}
