package cn.ninegame.genericframework.tools.dynamicload;

import android.content.Context;
import android.os.Build;
import android.text.TextUtils;
import android.util.Log;

import java.io.File;
import java.lang.reflect.Array;
import java.lang.reflect.Constructor;
import java.lang.reflect.Field;
import java.lang.reflect.InvocationTargetException;
import java.util.List;
import java.util.zip.ZipFile;

import cn.ninegame.genericframework.tools.ApkUtils;
import cn.ninegame.genericframework.tools.ReflectHelper;
import cn.ninegame.genericframework.tools.SoLoader;
import dalvik.system.DexFile;

import static cn.ninegame.genericframework.tools.SoLoader.LIB_DIR;

/**
 * Created by zhonghui on 2017-11-02.
 * 动态加载so库.
 */

public class NativeLibManager {

    public static String[] abis;

    public static void init(Context context){
       try{
           ZipFile zipFile = ApkUtils.getApk(context);
           SoLoader.extractSoFromApk(zipFile,abis);
           injectHackNativeLib(LIB_DIR,context.getClassLoader());
       }catch (Exception ex){
           ex.printStackTrace();
           Log.e("NativeLibManager",ex.getMessage().toString());
       }
    }

    /**
     * Inect so file into ClassLoader
     * @param patchSoFile
     * @param paramClassLoader
     * @throws ClassNotFoundException
     * @throws NoSuchMethodException
     * @throws NoSuchFieldException
     * @throws Exception
     */
    public static void injectHackNativeLib(File patchSoFile, ClassLoader paramClassLoader) throws ClassNotFoundException, NoSuchMethodException,NoSuchFieldException,Exception{
        Field pathListField = ReflectHelper.findField(paramClassLoader, "pathList");
        Object localPathList = pathListField.get(paramClassLoader);
        if ((!TextUtils.isEmpty(patchSoFile.getAbsolutePath())) && (patchSoFile.exists())) {
            Object newPathElement;
            if (Build.VERSION.SDK_INT >= 26) {
                newPathElement = makePathElementV26(patchSoFile);
                injectElementWithReplace(localPathList, "nativeLibraryPathElements", newPathElement);
                injectElementWithAdd(localPathList, "nativeLibraryDirectories", patchSoFile);
            } else if (Build.VERSION.SDK_INT >= 23) {
                newPathElement = makePathElementV23(patchSoFile);
                injectElementWithReplace(localPathList, "nativeLibraryPathElements", newPathElement);
                injectElementWithAdd(localPathList, "nativeLibraryDirectories", patchSoFile);
            } else {
                injectElementWithReplace(localPathList, "nativeLibraryDirectories", patchSoFile);
            }
        }
    }

    private static void injectElementWithReplace(Object paramObject1, String paramString, Object paramObject2) throws NoSuchFieldException,IllegalAccessException,NoSuchFieldException{
        Field localField = ReflectHelper.findField(paramObject1, paramString);
        Object[] arrayOfObject1 = (Object[]) (Object[]) localField.get(paramObject1);
        Object[] arrayOfObject2 = (Object[]) (Object[]) Array.newInstance(arrayOfObject1
                .getClass().getComponentType(), arrayOfObject1.length + 1);
        arrayOfObject2[0] = paramObject2;
        System.arraycopy(arrayOfObject1, 0, arrayOfObject2, 1, arrayOfObject1.length);
        localField.set(paramObject1, arrayOfObject2);
    }

    private static void injectElementWithAdd(Object paramObject1, String paramString, Object paramObject2) throws NoSuchFieldException,IllegalAccessException {
        Field localField = ReflectHelper.findField(paramObject1, paramString);
        List localList = (List) localField.get(paramObject1);
        localList.add(0, paramObject2);
    }

    private static Object makePathElementV23(File paramFile) throws ClassNotFoundException, NoSuchMethodException,InvocationTargetException,InstantiationException,IllegalAccessException,Exception {
        Class localClass = Class.forName("dalvik.system.DexPathList$Element");
        Class[] arrayOfClass = {File.class, Boolean.TYPE, File.class, DexFile.class};
        Constructor localConstructor = localClass.getDeclaredConstructor(arrayOfClass);
        if (localConstructor != null) {
            return localConstructor.newInstance(new Object[]{paramFile, Boolean.valueOf(true), null, null});
        }
        throw new Exception("make nativeElement fail no such constructor");
    }

    private static Object makePathElementV26(File paramFile) throws ClassNotFoundException, NoSuchMethodException,InvocationTargetException,InstantiationException,IllegalAccessException,Exception {
        Class localClass = Class.forName("dalvik.system.DexPathList$NativeLibraryElement");
        Class[] arrayOfClass = {File.class};
        Constructor localConstructor = localClass.getDeclaredConstructor(arrayOfClass);
        if (localConstructor != null) {
            localConstructor.setAccessible(true);
            return localConstructor.newInstance(new Object[]{paramFile});
        }
        throw new Exception("make nativeElement fail no such constructor");
    }

    private static Field getObjectField(Object paramObject, String paramString) throws NoSuchFieldException {
        Class localClass = paramObject.getClass();
        try {
            Field localField = localClass.getDeclaredField(paramString);
            if (!localField.isAccessible()) {
                localField.setAccessible(true);
            }
            return localField;
        } catch (NoSuchFieldException localNoSuchFieldException) {
            throw new NoSuchFieldException("Field " + paramString + " not found in " + paramObject.getClass());
        }
    }

    static {
        abis = new String[]{android.os.Build.CPU_ABI, android.os.Build.CPU_ABI2};
        /*
        try {
            Application localApplication = RuntimeVariables.androidApplication;
            PackageManager localPackageManager = localApplication.getPackageManager();
            if (localPackageManager != null) {
                ApplicationInfo localApplicationInfo = localPackageManager.getApplicationInfo(localApplication.getPackageName(), 0);
                if (localApplicationInfo != null) {
                    if (Build.VERSION.SDK_INT >= 21) {
                        abis = Build.SUPPORTED_ABIS;
                        if(abis==null){
                            Field localField = ReflectHelper.findField(localApplicationInfo, "primaryCpuAbi");
                            localField.setAccessible(true);
                            String str = (String) localField.get(localApplicationInfo);
                            if(!TextUtils.isEmpty(str)){
                                abis = new String[]{str};
                            }else{
                                abis = Build.SUPPORTED_ABIS;
                            }
                        }
                    } else {
                        abis = new String[]{Build.CPU_ABI, Build.CPU_ABI2};
                    }
                }
            }
        } catch (Throwable localThrowable) {
            localThrowable.printStackTrace();
            Log.e("NativeLibManager", "NativeLibManager static block " + localThrowable.getMessage());
        }*/
    }

}
